/*
 *  ofxFern.h
 *  
 *
 *  Created by theo on 16/12/2009.
 *  Copyright 2009. All rights reserved.
 *
 *	Updated by Dan Wilcox <danomatika@gmail.com> 2012
 */

#pragma once 

#include "ofMain.h"
#include "ofxOpenCv.h"
 //ADDITION AR BOOK
#include "ofxCv.h"

#include "mcv.h"
#include "planar_pattern_detector_builder.h"
#include "template_matching_based_tracker.h"

//ADDITION AR BOOK
#define FERN_PAGE_CENTER 0
#define FERN_PAGE_LEFT 1
#define FERN_PAGE_RIGHT 2

struct ofxFernTrackedMarker{

	ofxFernTrackedMarker() {
		tracked = false;
		calibration.load("kinect-color.yml");
	}

	void draw() {
		ofBeginShape();
			ofVertex(points[0]);
			ofVertex(points[1]);
			ofVertex(points[2]);
			ofVertex(points[3]);
		ofEndShape(true);
	}

	ofPoint points[4];	
	//ADDITION AR BOOK
	ofPoint pagePoints[4];

	//ADDITION AR BOOK
	void calculatePagePoints() {
		
		float pageWidth = (ofDist(points[0].x, points[0].y, points[1].x, points[1].y)+ ofDist(points[3].x, points[3].y, points[2].x, points[2].y))/2;//CALCULATE AVERAGE page width
		float gap = pageWidth*0.15;

		if (trackerPStyle == FERN_PAGE_LEFT) {

			ofVec3f calcPt = points[0] - points[1];
			calcPt.normalize();
			calcPt *= gap;
			calcPt += points[0];
			pagePoints[0] = calcPt;

			calcPt = points[0] - points[1];
			calcPt.normalize();
			calcPt *= pageWidth;
			calcPt += points[0];
			pagePoints[1] = calcPt;

			calcPt = points[3] - points[2];
			calcPt.normalize();
			calcPt *= pageWidth;
			calcPt += points[3];
			pagePoints[2] = calcPt;

			calcPt = points[3] - points[2];
			calcPt.normalize();
			calcPt *= gap;
			calcPt += points[3];
			pagePoints[3] = calcPt;
		
		}
		else if(trackerPStyle == FERN_PAGE_RIGHT){

			ofVec3f calcPt =points[1] - points[0]  ;
			calcPt.normalize();
			calcPt *= gap;
			calcPt += points[1];
			pagePoints[0] = calcPt;

			calcPt =  points[1] - points[0];
			calcPt.normalize();
			calcPt *= pageWidth;
			calcPt += points[1];
			pagePoints[1] = calcPt;

			calcPt = points[2] - points[3];
			calcPt.normalize();
			calcPt *= pageWidth;
			calcPt += points[2];
			pagePoints[2] = calcPt;

			calcPt = points[2] - points[3];
			calcPt.normalize();
			calcPt *= gap;
			calcPt += points[2];
			pagePoints[3] = calcPt;

		}



	}
	void setPageStyle(int style) {
		trackerPStyle = style;
	}
	ofPoint getCenter() {
	
		ofPoint triangleLeftAverage = (points[0] + points[1] + points[2])/3;
		ofPoint triangleRightAverage = (points[2] + points[3] + points[0])/3;	
		ofPoint average=(triangleLeftAverage + triangleRightAverage) / 2;
		return average;

	}
	ofPoint getOrientation(){
		return getOrientationPt(points);
	}


	ofPoint getCenterPage() {
		calculatePagePoints();
		ofPoint triangleLeftAverage = (pagePoints[0] + pagePoints[1] + pagePoints[2]) / 3;
		ofPoint triangleRightAverage = (pagePoints[2] + pagePoints[3] + pagePoints[0]) / 3;
		ofPoint average = (triangleLeftAverage + triangleRightAverage) / 2;
		if (trackerPStyle == 0) { cout << "page style not set" << endl; }
		return average;

	}
	ofPoint getOrientationPage() {
		calculatePagePoints();
		if (trackerPStyle == 0) { cout << "page style not set" << endl; }
		return getOrientationPt(pagePoints);
	}


	ofPoint getOrientationPt(ofPoint pts[4]) {

		//TODO try coplanar posit and other techniques
		//NOTE if you use different camera than the kinect 360 you will need to create calibration file for the camera (using ofxCv calibration camera exemple)

		vector<cv::Point2f> cornersp;
		cornersp.push_back(cv::Point2f(pts[0].x, pts[0].y));
		cornersp.push_back(cv::Point2f(pts[1].x, pts[1].y));
		cornersp.push_back(cv::Point2f(pts[2].x, pts[2].y));
		cornersp.push_back(cv::Point2f(pts[3].x, pts[3].y));


		float pageWidth = (ofDist(pts[0].x, pts[0].y, pts[1].x, pts[1].y) + ofDist(pts[2].x, pts[2].y, pts[3].x, pts[3].y)) / 2;
		float pageHeight = (ofDist(pts[0].x, pts[0].y, pts[3].x, pts[3].y) + ofDist(pts[1].x, pts[1].y, pts[2].x, pts[2].y)) / 2;

		vector<cv::Point3f> objp;
		objp.push_back(cv::Point3f(0, pageHeight,0));
		objp.push_back(cv::Point3f(0, 0, 0));
		objp.push_back(cv::Point3f(pageWidth, 0, 0));
		objp.push_back(cv::Point3f(pageWidth, pageHeight, 0));
	
		cv::Mat cameraMatrix = calibration.getDistortedIntrinsics().getCameraMatrix();
		cv::Mat distortion = calibration.getDistCoeffs();
		cv::Mat translation;
		cv::Mat rotation;
		cv::solvePnPRansac(objp, cornersp, cameraMatrix, distortion, rotation, translation, false);

		ofPoint euler(ofRadToDeg(rotation.at<double>(0)), ofRadToDeg(rotation.at<double>(1)), ofRadToDeg(rotation.at<double>(2)));
		return euler;
	}


	ofxCv::Calibration calibration;
	bool tracked;
	int trackerPStyle;
};

class ofxFern{

	public:
		
		ofxFern();
		~ofxFern();

		/// set the marker image file and incoming image size
		void setup(string marker, int width, int height);
		
		/// search for marker in an incoming image
		/// dimensions should match size in setup()
		void update(ofxCvGrayscaleImage & grayImg);
		
		/// reset the tracker, restarts detected next frame
		void resetTracker();
		
		/// the tracked marker
		ofxFernTrackedMarker trackedMarker;
		
		/// draw the tracker image
		void draw(float x, float y, float w, float h);
		
		/// show tracked locations in the tracker image?
		void showLocations(bool show)	{show_tracked_locations = show;}
		bool getShowLocations()			{return show_tracked_locations;}
		
		/// show keypoints in the tracker image?
		void showKeypoints(bool show)	{show_keypoints = show;}
		bool getShowKeypoints()			{return show_keypoints;}
		
	protected:
				
		void draw_quadrangle(IplImage * frame, int u0, int v0, int u1, int v1, int u2, int v2, int u3, int v3, CvScalar color, int thickness = 1);
		void draw_detected_position(IplImage * frame, planar_pattern_detector * detector);
		void draw_initial_rectangle(IplImage * frame, template_matching_based_tracker * tracker);
		void draw_tracked_position(IplImage * frame, template_matching_based_tracker * tracker);
		void draw_tracked_locations(IplImage * frame, template_matching_based_tracker * tracker);
		void draw_detected_keypoints(IplImage * frame, planar_pattern_detector * detector);
		void draw_recognized_keypoints(IplImage * frame, planar_pattern_detector * detector);
		
		void detect(IplImage * frame);
		
		void setTrackedMarker(template_matching_based_tracker * tracker);
		void setTrackedMarker(planar_pattern_detector * detector);

		bool bSetup;		
		ofxCvGrayscaleImage img;

		bool show_tracked_locations;
		bool show_keypoints;
		planar_pattern_detector * detector;
		template_matching_based_tracker * tracker;
		bool bDetected;	///< has the marker been detected?

};
