#define PI (3.1415926536)
#define TWO_PI (6.2831853072)

uniform float elapsedTime;
uniform float alpha;
uniform bool wrap;
uniform vec2 resolution;
varying vec3 position, normal;
varying float randomOffset;
uniform float val0;
uniform float val1;
uniform float val2;
uniform float val3;
uniform float val4;


const float Tau		= 6.2832;
//const float speed	= .02;
//const float density	= .04;
//const float shape	= .04;

float random(vec2 seed) {
    return fract(sin(seed.x + seed.y * 1e3) * 1e5);
}

float Cell(vec2 coord) {
    vec2 cell = fract(coord) * vec2(.5, 2.) - vec2(.1, .5);
    return (1. - length(cell * 2. - 1.)) * step(random(floor(coord)), val3) * 5.;
}

const float WAVE_LENGTH = 45.0; // pixels
const float FREQUENCY = 1.0; // this many rotations per second

float waveFrom(vec2 emitterPosition, vec2 position) {
	float k = 2.0 * PI / WAVE_LENGTH;
	float x = length(emitterPosition - position);
	float w = 2.0 * PI * FREQUENCY;
	float t = mod(elapsedTime, 1.0 / FREQUENCY);
	
	// wave equation is sin(kx - wt), where:
	// k is the wave-number ( 2PI / wavelength)
	// x is distance
	// w is angular frequency (2PI * frequency)
	// t is time
	#define waveEquation(k, x, w, t) (cos(k*x - w*t))
	
	return waveEquation((4./3.)/pow(k, 1.1+cos(length(vec2(val2,val3)-.5)*5.)), length(emitterPosition - position), w, t);
}

void main( void ) {


	float stages = 4.;
	float stage =round(val1*stages);//mod(elapsedTime * .6, stages);// 0.; //

	if(stage == 0.) {


	vec2 p = vec2( gl_FragCoord.xy / resolution.xy );
	p = 2.0 * p - 1.0;
	p.x *= resolution.x / resolution.y;
	
	
	float t = elapsedTime * 50.0* val3;
	vec3 color = vec3(0.0);

float pi=PI;

	float m1 = (1.0 - abs(atan(p.x - 0.25, p.y+1.0)) / PI * (10.5*val2));
	float m2 = (1.0 - abs(atan(p.x + 0.25, p.y+1.0)) / PI * (10.5*val2));

	float wave1 = cos(length(p - vec2(0.25, -1.0)) * PI * 60.0  - t) * smoothstep(0.0, 0.3, m1);
	float wave2 = cos(length(p - vec2(-0.25, -1.0)) * PI * 60.0  - t) * smoothstep(0.0, 0.3, m2);
	  

	vec3 bg = vec3(0.0);

		 bg = vec3(0.0, 0.0, 0.0);
	 color = mix(bg, vec3(1.0, 1.0, 1.0), wave1); 
	color = mix(color, vec3(1.0, 1.0, 1.0), wave2);

	gl_FragColor = vec4( color, 1.0 );
	} else if (stage < 2.){

gl_FragColor = vec4( 0.0,0.0,0.0, 1.0 );

vec2 uv = (gl_FragCoord.xy / resolution - 0.5) * vec2(resolution.x/resolution.y,1);
	vec3 ray = normalize(vec3(uv,0.9));
	vec3 p = vec3(0,0,fract(0.9 * elapsedTime*0.7));
	for(float i = 0.0; i < 100.0; i++){
		float d = 0.5 * distance(fract(2.0 * p),vec3(0.5));
		if(d < 0.195 * val4){
			
			// COLOR to mouse y
			gl_FragColor = vec4(vec3(1.0-float(i)/ val2),1.0);
			return;
		}
		
		
		p+= ray * 0.9;
	}

	// BG COLOR
	gl_FragColor = vec4(1.0,1.0,1.0,1.0);

	}

	else if (stage < 3.){




    vec2 p = gl_FragCoord.xy / resolution - vec2(0.5,0.5);
	p.x*=resolution.x/resolution.y; // fixed ratio ? :)

    float a = fract(atan(p.x, p.y) / Tau);
    float d = length(p);

    vec2 coord = vec2(pow(d, val2), a) * 256.;
    vec2 delta = vec2(-elapsedTime * val4 * 256., .5);

    float c = 0.;
    for (int i = 0; i < 8; i++) {
        coord += delta;
        c = max(c, Cell(coord));
    }

    gl_FragColor = vec4(vec3(c * d),1.0);

	}
	else if (stage < 4.){
vec2 position = gl_FragCoord.xy;//*val4
	vec2 emitterPosition = vec2(0.5);//*val4
	
	float height = 0.0;
	const int NUM_REFLECTIONS = 1.0;
	for(int x = -NUM_REFLECTIONS; x <= NUM_REFLECTIONS; x++) {
		for(int y = -NUM_REFLECTIONS; y <= NUM_REFLECTIONS; y++) {
			height += waveFrom((vec2(x,y) + emitterPosition) * resolution*val4, position*val4);
		}
	}
/*	float height = waveFrom(emitterPosition * resolution, position) 
		       + waveFrom(vec2(0.5, -0.5) * resolution, position)
		       + waveFrom(vec2(0.5, 1.5) * resolution, position)
		       + waveFrom(vec2(-0.5, 0.5) * resolution, position)
		       + waveFrom(vec2(1.5, 0.5) * resolution, position)
		       ;
*/
	
	float color = height / 2.0 / float(2*NUM_REFLECTIONS+1) + 0.5;

	gl_FragColor = vec4( vec3(color), 1.0 );
	}
gl_FragColor.a=alpha;
}