#pragma once

//BASED ON ofxProjectorKinectCalibration calibration exemple

#include "ofMain.h"
#include "ofxOpenCv.h"
#include "ofxCv.h"
#include "ofxFern.h"
#include "ofxUI.h"

#include "ofxKinect.h"
#include "RGBDCamCalibWrapperOfxKinect.h"
#include "ofxKinectProjectorCalibration.h"

#include "ofxXmlSettings.h"


#define PAGES 3 
#define CAMERA_W 640
#define CAMERA_H 480
#define FIRST_SCREEN_W 1920
#define FIRST_SCREEN_H 1080
#define PROJ_W 1280
#define PROJ_H 800

#include "scene1.h"
#include "scene2.h"
#include "scene3.h"



using namespace cv;

class ofApp : public ofBaseApp{

	public:
		void setup();

		void update();
		void draw();
		void drawProj(ofEventArgs & args);
		void drawScene(int sceneNum);
		void updateScene(int sceneNum);
		void exit();

		void keyPressed(int key);
		void keyReleased(int key);
		void mouseMoved(int x, int y );
		void mouseDragged(int x, int y, int button);
		void mousePressed(int x, int y, int button);
		void mouseReleased(int x, int y, int button);
		void windowResized(int w, int h);
		void dragEvent(ofDragInfo dragInfo);
		void gotMessage(ofMessage msg);
    
    // gui
    void setupGui();
    ofxUISuperCanvas *          gui;
    ofxUISuperCanvas *			guiImageSettings;
	ofxUISuperCanvas *			guiAR;
    void guiEvent(ofxUIEventArgs &e);
    void guiUpdateLabels();
    //ofxPanel gui;

private:
        
        // kinect & the wrapper
        
        ofxKinect                   kinect;
        ofxCvColorImage				kinectColorImage;
        ofxCvGrayscaleImage			kinectDepthImage;
        
        RGBDCamCalibWrapper*		kinectWrapper;
        
        // calibration
        KinectProjectorCalibration	kinectProjectorCalibration;
        bool						enableCalibration;
        ofxCv::ContourFinder        contourFinder;
        
        // output
        KinectProjectorOutput		kinectProjectorOutput;
        bool						enableTestmode;
        float                       chessboardThreshold;
        ofImage						thresholdedKinect;
        float                       lowThresh;
        float                       highThresh;
        int							blur;
        float                       maxReprojError;

		// output
		bool						enableARmode;

    
        // settings
        int                         projectorWidth;
        int                         projectorHeight;


		int kWidth;
		int kHeight;
        
        ofParameterGroup labels;

		ofxCvColorImage colorImg;
		ofxCvGrayscaleImage img;
		vector<ofxFern>fern;
		vector<ofVec3f>rotationMarkers;
		vector<ofVec3f>positionMarkers;
		bool drawDebug;
		ofVec3f posLight;
		ofVec3f rotationLight;
		float opacityLight;
		float shaderVal[5];

		bool showTracker;

		//TODO use a scene manager
		scene1 scene1Demo;
		scene2 scene2Demo;
		scene3 scene3Demo;

		ofImage light;
		float opacityMarker;

};
